package fullenv

import (
	"regexp"
	"strings"
	"testing"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/testutils"

	"github.com/stretchr/testify/assert"
)

// TestGraphReload makes sure a freshly activated graph matches incoming
// envelopes as it should
func TestGraphReload(t *testing.T) {
	env, log := NewFullEnvForTest(t, "graphreload", "graphreload.yaml")

	if err := env.Init(); err != nil {
		t.Fatal(err)
	}

	defer env.Close()
	log.ExpectsError(
		testutils.ExpectsContains("Error sending envelope: No matching graph"))

	if err := env.Startup(); err != nil {
		t.Fatal(err)
	}

	sender := env.Emitter.Actor.OpenOutput(api.EnvelopeContext{}, "default", "demo.count")
	must(sender.AddItems("demo.count", api.Item("item")))
	err := sender.Close()
	if err == nil {
		t.Fatal("emit should fail with 'No matching graph'")
	}
	if !strings.HasSuffix(err.Error(), "No matching graph") {
		t.Fatal(err)
	}

	// now activate the right version of the graph
	env.XbusCtl.MustRun("graph", "status", "demo", "good", "--active")

	sender = env.Emitter.Actor.OpenOutput(api.EnvelopeContext{}, "default", "demo.count")
	must(sender.AddItems("demo.count", api.Item("item")))
	must(sender.Close())

	re, err := regexp.Compile("\\| ([0-9a-f-]+) \\| [0-9a-f-]+ \\| [0-9a-f-]+ \\| Process[a-zA-Z]+ \\|") // EXTRACT PIDS)
	must(err)

	matchLines, err := env.XbusCtl.RunReadAndFindSubmatch(
		re, "ps", "--status-running")
	must(err)
	assert.Equal(t, 0, len(matchLines))

	matchLines, err = env.XbusCtl.RunReadAndFindSubmatch(
		re, "ps", "--status-error")
	must(err)
	assert.Equal(t, 1, len(matchLines))

	matchLines, err = env.XbusCtl.RunReadAndFindSubmatch(
		re, "ps", "--status-done")
	must(err)
	assert.Equal(t, 1, len(matchLines))
}
