package api

import (
	"github.com/m4rw3r/uuid"
)

const (
	// ControlPMProcess api to manage postmortem processes
	ControlPMProcess = ControlPrefix + "pm."
)

// PostMortemStatus ...
//go:generate enum-marshaler -type PostMortemStatus -prefix PostMortem -lower -tests
type PostMortemStatus int

const (
	// PostMortemDefault - the process is not noticeable
	PostMortemDefault PostMortemStatus = iota
	// PostMortemNew - the process is noticeable and was not acknowleged
	PostMortemNew
	// PostMortemOpened - someone is working on the process
	PostMortemOpened
	// PostMortemClosed - the process was reviewed and issue closed
	PostMortemClosed
)

// PMProcess is a process with postmortem information
type PMProcess struct {
	Process
	Level   LogLevel
	Status  PostMortemStatus
	Comment string
	Logs    []LogEntry
}

const (
	// ControlPMProcessQuery is the subject for postmortem process queries
	ControlPMProcessQuery = ControlPMProcess + "query"
)

// ControlPMProcessQueryReq is the args for ControlPMProcessQuery requests
type ControlPMProcessQueryReq struct {
	Level         LogLevel
	IncludeClosed bool
	ProcessIDs    []uuid.UUID

	ReturnLogs bool
}

// ControlPMProcessQueryRep is a reply
type ControlPMProcessQueryRep struct {
	Response
	PMProcessList []PMProcess
}

const (
	// ControlPMProcessSetStatus is the subject for setting a process postmortem status
	ControlPMProcessSetStatus = ControlPMProcess + "setstatus"
)

// ControlPMProcessSetStatusReq is the args for ControlPMProcessSetStatus
type ControlPMProcessSetStatusReq struct {
	ProcessID uuid.UUID
	Status    PostMortemStatus
	Comment   string
}
