package demo

import (
	"fmt"
	"time"

	"bitbucket.org/orus-io/go-xbus"
	"bitbucket.org/orus-io/go-xbus/api"
)

// NewHelloWorldEmitter returns a HelloWorldEmitter ActorService
// It is registered as "demo.helloworld" in the ActorService registry
func NewHelloWorldEmitter(actor *xbus.Actor) xbus.ActorService {
	return &HelloWorldEmitter{
		actor:    actor,
		message:  actor.MustGetStringSetting("message"),
		interval: int(actor.GetIntSettingD("interval", 1)),
	}
}

// HelloWorldEmitter sends 'demo.simplemessage' events periodically
type HelloWorldEmitter struct {
	actor    *xbus.Actor
	message  string
	interval int
	ticker   *time.Ticker
	quit     chan bool
}

// Startup initialize the ticker
func (e *HelloWorldEmitter) Startup() error {
	e.ticker = time.NewTicker(time.Duration(e.interval) * time.Second)
	e.quit = make(chan bool)
	go func() {
		count := 0
		for {
			select {
			case <-e.ticker.C:
				count++
				message := fmt.Sprintf("%s - %d", e.message, count)
				e.actor.Log.Noticef("Sending message '%s'", message)
				if _, err := e.actor.SendItems("default", "demo.simplemessage", []api.Item{api.Item(message)}); err != nil {
					e.actor.GetClient().Log.Errorf("send failed: %s", err)
				}
			case <-e.quit:
				return
			}
		}
	}()
	return nil
}

// Shutdown stops the ticker
func (e *HelloWorldEmitter) Shutdown() error {
	e.ticker.Stop()
	e.ticker = nil
	e.quit <- true
	return nil
}

func init() {
	xbus.RegisterActorService("demo.helloworld", NewHelloWorldEmitter)
}
